#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: create.pl,v 1.0.0.6 Mon 14 Apr 2008 06:35:19 PM CEST mstauber Exp $
# Copyright 2006-2008 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.
#
# Handler which creates new VPS's.
#

use CCE;
use POSIX qw(isalpha);

$DEBUG = 0;

my $cce = new CCE('Domain' => 'vz-vserver');

$cce->connectfd();

# Double check if the IP is not already assigned - kinda redundant, but never mind:

$vserver = $cce->event_object();
$vserver_new = $cce->event_new();
$vserver_old = $cce->event_old();

@oids= $cce->find("Vserver");
($ok, $system) = $cce->get($oids[0]);

$vpsid = $vserver_new->{vpsid};

# De-Scalarize $NAMESERVER:
$NAMESERVER = $vserver_new->{NAMESERVER};
@dns = split('&', $NAMESERVER);
shift(@dns);
$nameservers = join(" ", @dns);

if ($DEBUG == 1) {
    system("echo \"VPSID-debug: controlAction: $vserver_new->{controlAction}\" > /tmp/vps_create");
    system("echo \"VPSID-debug: vpsid: $vserver_new->{vpsid}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: vpsid: $vpsid\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: OSTEMPLATE: $vserver_new->{OSTEMPLATE}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: ORIGIN_SAMPLE: $vserver_new->{ORIGIN_SAMPLE}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: IP: $vserver_new->{ip}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: Hostname: $vserver_new->{hostname}\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: DNS: $nameservers\" >> /tmp/vps_create");
    system("echo \"VPSID-debug: Onboot: $vserver_new->{ONBOOT}\" >> /tmp/vps_create");
}

# Check if VPS IP-Address causes conflict:
if (($vserver_new->{ip}) || ($vserver->{ip}) || ($vserver_old->{ip})) {

    if ($vserver->{ip} ne '') { $validate_ip = $vserver->{ip}; }
    if ($vserver_old->{ip} ne '') { $validate_ip = $vserver_old->{ip}; }
    if ($vserver_new->{ip} ne '') { $validate_ip = $vserver_new->{ip}; }

        @oids = $cce->findx("Vserver", {},
                               {
                                'ip' => $validate_ip
                               });

        # there should be no oids found
        if (scalar(@oids) > 1) {
                $the_scalar = scalar(@oids);
                $out_msg = $oids[0];
                $cce->bye('FAIL',
                          "[[vz-vserver.ipInUse,ip='$validate_ip (CCE Object ID: $out_msg - $the_scalar)']]");
                exit(1);
        }

        # verify that the VPS doesn't have the same IP as the master node itself:
        my ($network) = $cce->find('Network', { 'ipaddr' => $validate_ip, 'enabled' => '1' });
        if ($network) {
                $cce->bye('FAIL',
                          "[[vz-vserver.ipInUseMasterNode,ip='$validate_ip (CCE Object ID: $network)']]");
                exit(1);
        }

        # Also check vpsIPalloc for IPs that may be assigned as 2ndary IPs to other VPSs:
        @oids = $cce->findx("vpsIPalloc",
                               {
                                'min' => $validate_ip
                               });

        # there should be no oids found
        if (scalar(@oids) > 0) {
                $the_scalar = scalar(@oids);
                foreach $line (@oids) {
                        $out_msg = $out_msg . " " . $line;
                }

                $cce->bye('FAIL',
                          "[[vz-vserver.ipInUse,ip='$validate_ip (CCE Object ID:$out_msg - $the_scalar)']]");
                exit(1);
        }
}

if (($vserver_new->{controlAction} eq "create") && ($vpsid) && ($vserver_new->{ORIGIN_SAMPLE}) && ($vserver_new->{OSTEMPLATE})) {
    if ($DEBUG == 1) {
	system("echo /usr/sbin/vzctl create $vpsid --ostemplate $vserver_new->{OSTEMPLATE} --config $vserver_new->{ORIGIN_SAMPLE} >> /tmp/vps_create");
    }
    if ($vserver_new->{ONBOOT} eq "Yes") {
	$onboot_param = "yes";
    }
    else {
	$onboot_param = "no";
    }
    system("/usr/sbin/vzctl create $vpsid --ostemplate $vserver_new->{OSTEMPLATE} --config $vserver_new->{ORIGIN_SAMPLE}");
    system("/usr/sbin/vzctl set $vpsid --nameserver \"$nameservers\" --hostname $vserver_new->{hostname} --ipadd $vserver_new->{ip} --onboot $onboot_param --save");

    # If - at this point - we have something like "/vz/private/$vpsid.tmp" left around, then the VPS could not be created.
    # Possible causes are a corruption of the OS template - or more likely: VPS needed more space than granted in the sample
    # config file that was used. So exit with error code 1 and CCE msg "failOnCreate".

    if (-d "/vz/private/$vpsid.tmp") {
        $cce->bye('FAIL', "[[vz-vserver.failOnCreate]]");
        exit(1);
    }
    else {
        system("/bin/chmod 755 /vz/private/$vpsid");
        system("/bin/chmod 644 /etc/vz/conf/$vpsid.conf");
        system("/usr/sbin/vzctl start $vpsid");
        #system("/usr/sausalito/handlers/vz/vserver/delayed_network_restart.pl vpsid $vpsid &");
    }
    #&cleanup;
}

sub cleanup {
    # Control action may also need to be reset to 'none' once we're done - but not on delete!
    if ($vserver_new->{controlAction}) {
	($sys_oid) = $cce->find('Vserver', {'vpsid' => $vpsid});
	($ok, $sysvs) = $cce->get($sys_oid);
	($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
	    'vpsid' => $vpsid,
	    'controlAction' => 'none'
	});
    }
}

# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);


