#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: disk_mountpoints.pl, v1.1.0.5 Fri 28 Mar 2008 11:44:59 PM MDT mstauber Exp $
# Copyright 2006-2008 Solarspeed Ltd. All rights reserved.

# This constructor / handler checks which mountpoints are available to store the local
# backups.

use CCE;
my $cce = new CCE;
my $conf = '/var/lib/cobalt';

$cce->connectfd();

# Location of the vzlist-dump:
$diskdump = '/tmp/vserver-disk';

# We are starting sane:
$bad_info_detected = '0';

# Destroy all previous Vserver_Disks CCE entries:
&diskwipe;

# Parse existing data and shove it into CCE:
&main;

$cce->bye('SUCCESS');
exit(0);

##
# Subs:
##

sub diskwipe {

        # Find all previous Vserver_Disks CCE entries and destroy them:
        @oldsamples = $cce->findx('Vserver_Disks');
        for $entry (@oldsamples) {
            ($ok) = $cce->destroy($entry);
        }
}

sub main {

    # Generate disk-dump:
    if ( -e "/dev/mapper/VolGroup00-LogVol00" ) {
        system("/bin/df -h |/bin/grep -v tmpfs|/bin/grep -v ^Filesystem|/bin/grep -v none|/bin/grep -v /boot|/bin/grep -v /tmp |/bin/grep -v /var | /bin/grep -v /dev/mapper/ |/bin/sed 's/^[ \t]*/\\/dev\\/mapper\\/VolGroup00 /;s/[ \t]*\$//' | /bin/grep -v /var > $diskdump");
    }
    else {
        system("/bin/df -h |/bin/grep -v tmpfs|/bin/grep -v ^Filesystem|/bin/grep -v none|/bin/grep -v /boot|/bin/grep -v /tmp |/bin/grep -v /var | /bin/grep -v /var > $diskdump");
    }

    # Pull the vzlist-dump into a Matrix:
    #
    # I LOVE matrix stuff! If there is a sledload of data to parse, 
    # then there is hardly a more efficient way to do so.
    # A legend of what colums the matrix has is at the end of the code.

    open (F, $diskdump) || die "Could not open $diskdump: $!";
    $a = 0; 					# Matrix line number, whereas 0 is the first (useable) line of the vzlist dump
    $sambashare = 0;
    while ($line = <F>) {
	chomp($line);
	next if $line =~ /^\s*$/; 		# skip blank lines
        $matrix_name = 'DISKINFO';
        my (@row) = split (/\s+/, $line);
        push (@{$matrix_name}, \@row); 
    	$currentdisk = $DISKINFO[$a][0];
        if ($DISKINFO[$a][0] =~ /dev/) { 		# Plausibility check. 
	    &validate;
	    &feedthemonster;		# Feed info into CCE
	}
	else {
	    # Handle Samba-Shares:
	    if ($currentdisk =~ /^\/\/(.*)/) {
		$previousdisk = $currentdisk;
		$sambashare = 1;
	    }
	    if (($currentdisk eq "") && ($sambashare == "1")) {
		$DISKINFO[$a][0] = $previousdisk;
		&validate;
		&feedthemonster;            # Feed info into CCE
		$sambashare = 0;
	    }
	}
	$a++;
    }
    close(F);
    system("/bin/rm $diskdump");

}

# Subroutine that feeds the data into CCE:
sub feedthemonster {
    if (($DISKINFO[$a][0] ne "") && ($DISKINFO[$a][6] == "1") && ($DISKINFO[$a][5] ne "")) {

	# This is where we actually populate CCE with the gathered info:
        @oids = $cce->find('Vserver_Disks', {'physical' => $currentdisk});
        if ($#oids < 0) {
                #print "Object not yet in CCE. Creating new one.\n";
                ($ok) = $cce->create('Vserver_Disks', {
                'physical' => $DISKINFO[$a][0],
                'size' => $DISKINFO[$a][1],
                'used' => $DISKINFO[$a][2],
                'avail' => $DISKINFO[$a][3],
                'used_percent' => $DISKINFO[$a][4],
                'mountpoint' => $DISKINFO[$a][5],
                'writeable' => $DISKINFO[$a][6],
                'timestamp' => time()
                });
		$new_disk_found = '1'; 			# We found a new Disk
        }
        else {
	        #print "Object already present in CCE. Updating it.\n";
                # find the system object
		$new_disk_found = '0';
                ($sys_oid) = $cce->find('Vserver_Disks', {'physical' => $currentdisk});
                ($ok, $sys) = $cce->get($sys_oid);
		
                ($ok) = $cce->set($sys_oid, '',{ 	# beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
                'physical' => $DISKINFO[$a][0],
                'size' => $DISKINFO[$a][1],
                'used' => $DISKINFO[$a][2],
                'avail' => $DISKINFO[$a][3],
                'used_percent' => $DISKINFO[$a][4],
                'mountpoint' => $DISKINFO[$a][5],
                'writeable' => $DISKINFO[$a][6],
                'timestamp' => time()
                });
	}
    }
}


# Validate if Disk is writeable before we write the info into CCE:
sub validate {

	if (($DISKINFO[$a][0]) && ($DISKINFO[$a][5])) {

	    if ( -w $DISKINFO[$a][5] ) {
		$status = "writeable";
		$DISKINFO[$a][6] = "1";
	    }
	    else {
		$status = "not writeable";
		$DISKINFO[$a][6] = "0";
	    }
	}
}

$cce->bye('SUCCESS');
exit(0);


