#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: make_sense.pl,v 1.0.0.6 Mon 14 Apr 2008 06:36:58 PM CEST mstauber Exp $
# Copyright 2006-2008 Solarspeed Ltd. All rights reserved.
#
# Run checks on the values passed through the GUI to make sure that they make sense.
#

use CCE;
use POSIX qw(isalpha);

$DEBUG = 0;

#$cce = new CCE('Domain' => 'vz-vserver');
$cce = new CCE;

$cce->connectfd();
#$cce->connectuds(); # only for debugging

$vserver = $cce->event_object();
$vserver_new = $cce->event_new();
$vserver_old = $cce->event_old();

@oids= $cce->find("Vserver");
($ok, $system) = $cce->get($oids[0]);

# Check if VPS IP-Address causes conflict:
if (($vserver_new->{ip}) || ($vserver->{ip}) || ($vserver_old->{ip})) {

    if ($vserver->{ip} ne '') { $validate_ip = $vserver->{ip}; }
    if ($vserver_old->{ip} ne '') { $validate_ip = $vserver_old->{ip}; }
    if ($vserver_new->{ip} ne '') { $validate_ip = $vserver_new->{ip}; }

        @oids = $cce->findx("Vserver", {},
                               {
                                'ip' => $validate_ip
                               });

        # there should be no oids found
        if (scalar(@oids) > 1) {
                $the_scalar = scalar(@oids);
                $out_msg = $oids[0];
                $cce->bye('FAIL',
                          "[[vz-vserver.ipInUse,ip='$validate_ip (CCE Object ID: $out_msg - $the_scalar)']]");
                exit(1);
        }

        # verify that the VPS doesn't have the same IP as the master node itself:
        my ($network) = $cce->find('Network', { 'ipaddr' => $validate_ip, 'enabled' => '1' });
        if ($network) {
                $cce->bye('FAIL',
                          "[[vz-vserver.ipInUseMasterNode,ip='$validate_ip (CCE Object ID: $network)']]");
                exit(1);
        }

        # Also check vpsIPalloc for IPs that may be assigned as 2ndary IPs to other VPSs:
        @oids = $cce->findx("vpsIPalloc",
                               {
                                'min' => $validate_ip
                               });

        # there should be no oids found
        if (scalar(@oids) > 0) {
                $the_scalar = scalar(@oids);
                foreach $line (@oids) {
                        $out_msg = $out_msg . " " . $line;
                }

                $cce->bye('FAIL',
                          "[[vz-vserver.ipInUse,ip='$validate_ip (CCE Object ID:$out_msg - $the_scalar)']]");
                exit(1);
        }
}

# hostname must be less than or equal to 255
if ((length($vserver->{hostname}) > 255) ||
    (length("$vserver->{hostname}") > 255)) {

    $cce->bye('FAIL', "[[vz-vserver.hostnameTooLong]]");
    exit(1);
}

# CPU-Units check:
$cpuunits = $vserver_new->{cpuunits};
$cpuunits_old = $vserver->{cpuunits};
$cpuunits_node = $vserver->{cpuunits_node};
$cpuunits_alloc = $vserver->{cpuunits_alloc};

if (($cpuunits_node) && ($cpuunits > $cpuunits_node)) { # $cpuunits_node may not be set yet on first run of constructor
    $cce->bye('FAIL', "[[vz-vserver.cpuunitsnode]]");
        exit(1);
}    

if (($cpuunits_node) && ($cpuunits > $cpuunits_node-$cpuunits_alloc)) { # $cpuunits_node may not be set yet on first run of constructor
    $cce->bye('FAIL', "[[vz-vserver.cpuunitsnodeavail]]");
        exit(1);
}    

### Messer =!= Sau

# avnumproc:
if (($vserver_new->{avnumproc_b}) || ($vserver_new->{avnumproc_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{avnumproc_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{avnumproc_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{avnumproc_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{avnumproc_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{avnumproc_b} > $vserver_old->{avnumproc_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.avnumproc_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{avnumproc_b} > $vserver_new->{avnumproc_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.avnumproc_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{avnumproc_b} > $vserver_new->{avnumproc_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.avnumproc_mismatch]]");
    	    exit(1);
    }
}

# numproc:
if (($vserver_new->{numproc_b}) || ($vserver_new->{numproc_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{numproc_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{numproc_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{numproc_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{numproc_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{numproc_b} > $vserver_old->{numproc_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numproc_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{numproc_b} > $vserver_new->{numproc_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numproc_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{numproc_b} > $vserver_new->{numproc_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numproc_mismatch]]");
    	    exit(1);
    }
}

# numtcpsock:
if (($vserver_new->{numtcpsock_b}) || ($vserver_new->{numtcpsock_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{numtcpsock_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{numtcpsock_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{numtcpsock_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{numtcpsock_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{numtcpsock_b} > $vserver_old->{numtcpsock_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numtcpsock_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{numtcpsock_b} > $vserver_new->{numtcpsock_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numtcpsock_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{numtcpsock_b} > $vserver_new->{numtcpsock_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numtcpsock_mismatch]]");
    	    exit(1);
    }
}

# numothersock
if (($vserver_new->{numothersock_b}) || ($vserver_new->{numothersock_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{numothersock_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{numothersock_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{numothersock_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{numothersock_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{numothersock_b} > $vserver_old->{numothersock_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numothersock_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{numothersock_b} > $vserver_new->{numothersock_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numothersock_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{numothersock_b} > $vserver_new->{numothersock_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numothersock_mismatch]]");
    	    exit(1);
    }
}

# vmguarpages
if (($vserver_new->{vmguarpages_b}) || ($vserver_new->{vmguarpages_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{vmguarpages_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{vmguarpages_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{vmguarpages_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{vmguarpages_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{vmguarpages_b} > $vserver_old->{vmguarpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.vmguarpages_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{vmguarpages_b} > $vserver_new->{vmguarpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.vmguarpages_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{vmguarpages_b} > $vserver_new->{vmguarpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.vmguarpages_mismatch]]");
    	    exit(1);
    }
}

# kmemsize
if (($vserver_new->{kmemsize_b}) || ($vserver_new->{kmemsize_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{kmemsize_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{kmemsize_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{kmemsize_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{kmemsize_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{kmemsize_b} > $vserver_old->{kmemsize_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.kmemsize_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{kmemsize_b} > $vserver_new->{kmemsize_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.kmemsize_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{kmemsize_b} > $vserver_new->{kmemsize_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.kmemsize_mismatch]]");
    	    exit(1);
    }
}

# tcpsndbuf
if (($vserver_new->{tcpsndbuf_b}) || ($vserver_new->{tcpsndbuf_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{tcpsndbuf_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{tcpsndbuf_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{tcpsndbuf_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{tcpsndbuf_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{tcpsndbuf_b} > $vserver_old->{tcpsndbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.tcpsndbuf_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{tcpsndbuf_b} > $vserver_new->{tcpsndbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.tcpsndbuf_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{tcpsndbuf_b} > $vserver_new->{tcpsndbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.tcpsndbuf_mismatch]]");
    	    exit(1);
    }
}

# tcprcvbuf
if (($vserver_new->{tcprcvbuf_b}) || ($vserver_new->{tcprcvbuf_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{tcprcvbuf_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{tcprcvbuf_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{tcprcvbuf_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{tcprcvbuf_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{tcprcvbuf_b} > $vserver_old->{tcprcvbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.tcprcvbuf_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{tcprcvbuf_b} > $vserver_new->{tcprcvbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.tcprcvbuf_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{tcprcvbuf_b} > $vserver_new->{tcprcvbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.tcprcvbuf_mismatch]]");
    	    exit(1);
    }
}

# othersockbuf
if (($vserver_new->{othersockbuf_b}) || ($vserver_new->{othersockbuf_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{othersockbuf_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{othersockbuf_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{othersockbuf_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{othersockbuf_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{othersockbuf_b} > $vserver_old->{othersockbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.othersockbuf_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{othersockbuf_b} > $vserver_new->{othersockbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.othersockbuf_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{othersockbuf_b} > $vserver_new->{othersockbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.othersockbuf_mismatch]]");
    	    exit(1);
    }
}

# dgramrcvbuf
if (($vserver_new->{dgramrcvbuf_b}) || ($vserver_new->{dgramrcvbuf_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{dgramrcvbuf_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{dgramrcvbuf_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{dgramrcvbuf_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{dgramrcvbuf_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{dgramrcvbuf_b} > $vserver_old->{dgramrcvbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.dgramrcvbuf_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{dgramrcvbuf_b} > $vserver_new->{dgramrcvbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.dgramrcvbuf_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{dgramrcvbuf_b} > $vserver_new->{dgramrcvbuf_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.dgramrcvbuf_mismatch]]");
    	    exit(1);
    }
}

# oomguarpages
if (($vserver_new->{oomguarpages_b}) || ($vserver_new->{oomguarpages_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{oomguarpages_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{oomguarpages_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{oomguarpages_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{oomguarpages_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{oomguarpages_b} > $vserver_old->{oomguarpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.oomguarpages_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{oomguarpages_b} > $vserver_new->{oomguarpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.oomguarpages_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{oomguarpages_b} > $vserver_new->{oomguarpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.oomguarpages_mismatch]]");
    	    exit(1);
    }
}

# lockedpages
if (($vserver_new->{lockedpages_b}) || ($vserver_new->{lockedpages_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{lockedpages_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{lockedpages_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{lockedpages_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{lockedpages_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{lockedpages_b} > $vserver_old->{lockedpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.lockedpages_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{lockedpages_b} > $vserver_new->{lockedpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.lockedpages_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{lockedpages_b} > $vserver_new->{lockedpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.lockedpages_mismatch]]");
    	    exit(1);
    }
}

# shmpages
if (($vserver_new->{shmpages_b}) || ($vserver_new->{shmpages_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{shmpages_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{shmpages_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{shmpages_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{shmpages_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{shmpages_b} > $vserver_old->{shmpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.shmpages_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{shmpages_b} > $vserver_new->{shmpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.shmpages_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{shmpages_b} > $vserver_new->{shmpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.shmpages_mismatch]]");
    	    exit(1);
    }
}

# privvmpages
if (($vserver_new->{privvmpages_b}) || ($vserver_new->{privvmpages_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{privvmpages_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{privvmpages_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{privvmpages_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{privvmpages_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{privvmpages_b} > $vserver_old->{privvmpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.privvmpages_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{privvmpages_b} > $vserver_new->{privvmpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.privvmpages_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{privvmpages_b} > $vserver_new->{privvmpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.privvmpages_mismatch]]");
    	    exit(1);
    }
}

# numfile
if (($vserver_new->{numfile_b}) || ($vserver_new->{numfile_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{numfile_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{numfile_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{numfile_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{numfile_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{numfile_b} > $vserver_old->{numfile_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numfile_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{numfile_b} > $vserver_new->{numfile_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numfile_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{numfile_b} > $vserver_new->{numfile_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numfile_mismatch]]");
    	    exit(1);
    }
}

# numflock
if (($vserver_new->{numflock_b}) || ($vserver_new->{numflock_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{numflock_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{numflock_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{numflock_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{numflock_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{numflock_b} > $vserver_old->{numflock_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numflock_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{numflock_b} > $vserver_new->{numflock_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numflock_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{numflock_b} > $vserver_new->{numflock_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numflock_mismatch]]");
    	    exit(1);
    }
}

# numpty
if (($vserver_new->{numpty_b}) || ($vserver_new->{numpty_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{numpty_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{numpty_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{numpty_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{numpty_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{numpty_b} > $vserver_old->{numpty_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numpty_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{numpty_b} > $vserver_new->{numpty_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numpty_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{numpty_b} > $vserver_new->{numpty_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numpty_mismatch]]");
    	    exit(1);
    }
}

# numsiginfo
if (($vserver_new->{numsiginfo_b}) || ($vserver_new->{numsiginfo_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{numsiginfo_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{numsiginfo_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{numsiginfo_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{numsiginfo_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{numsiginfo_b} > $vserver_old->{numsiginfo_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numsiginfo_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{numsiginfo_b} > $vserver_new->{numsiginfo_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numsiginfo_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{numsiginfo_b} > $vserver_new->{numsiginfo_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numsiginfo_mismatch]]");
    	    exit(1);
    }
}

# dcachesize
if (($vserver_new->{dcachesize_b}) || ($vserver_new->{dcachesize_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{dcachesize_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{dcachesize_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{dcachesize_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{dcachesize_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{dcachesize_b} > $vserver_old->{dcachesize_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.dcachesize_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{dcachesize_b} > $vserver_new->{dcachesize_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.dcachesize_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{dcachesize_b} > $vserver_new->{dcachesize_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.dcachesize_mismatch]]");
    	    exit(1);
    }
}

# physpages
if (($vserver_new->{physpages_b}) || ($vserver_new->{physpages_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{physpages_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{physpages_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{physpages_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{physpages_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{physpages_b} > $vserver_old->{physpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.physpages_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{physpages_b} > $vserver_new->{physpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.physpages_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{physpages_b} > $vserver_new->{physpages_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.physpages_mismatch]]");
    	    exit(1);
    }
}

# numiptent
if (($vserver_new->{numiptent_b}) || ($vserver_new->{numiptent_l})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{numiptent_b} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{numiptent_b} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{numiptent_l} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{numiptent_l} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{numiptent_b} > $vserver_old->{numiptent_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numiptent_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{numiptent_b} > $vserver_new->{numiptent_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numiptent_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{numiptent_b} > $vserver_new->{numiptent_l})) {
	    $cce->bye('FAIL', "[[vz-vserver.numiptent_mismatch]]");
    	    exit(1);
    }
}

# diskspace
if (($vserver_new->{diskspace_s}) || ($vserver_new->{diskspace_h})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{diskspace_s} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{diskspace_s} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{diskspace_h} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{diskspace_h} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{diskspace_s} > $vserver_old->{diskspace_h})) {
	    $cce->bye('FAIL', "[[vz-vserver.diskspace_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{diskspace_s} > $vserver_new->{diskspace_h})) {
	    $cce->bye('FAIL', "[[vz-vserver.diskspace_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{diskspace_s} > $vserver_new->{diskspace_h})) {
	    $cce->bye('FAIL', "[[vz-vserver.diskspace_mismatch]]");
    	    exit(1);
    }
}

# diskinodes
if (($vserver_new->{diskinodes_s}) || ($vserver_new->{diskinodes_h})) {
    # Find out what's new - barrier, limit or both:
    if ($vserver_new->{diskinodes_s} > 0) { 
	$bar_new = 1; 
    }
    else {
	$bar_new = 0; 
    }	
    if ($vserver_old->{diskinodes_s} > 0) { 
	$bar_old = 1; 
    }
    else {
	$bar_old = 0; 
    }	
    if ($vserver_new->{diskinodes_h} > 0) { 
	$lim_new = 1; 
    }
    else {
	$lim_new = 0; 
    }
    if ($vserver_old->{diskinodes_h} > 0) { 
	$lim_old = 1; 
    }
    else {
	$lim_old = 0; 
    }
    # Case new : old
    if ((($bar_new == 1) && ($lim_old == 1) && ($lim_new == 0)) && ($vserver_new->{diskinodes_s} > $vserver_old->{diskinodes_h})) {
	    $cce->bye('FAIL', "[[vz-vserver.diskinodes_mismatch]]");
    	    exit(1);
    }
    # Case new : new 
    if ((($bar_new == 1) && ($bar_old == 1) && ($lim_old == 1) && ($lim_new == 1)) && ($vserver_new->{diskinodes_s} > $vserver_new->{diskinodes_h})) {
	    $cce->bye('FAIL', "[[vz-vserver.diskinodes_mismatch]]");
    	    exit(1);
    }
    # Case old : new
    if ((($bar_new == 0) && ($bar_old == 1) && ($lim_new == 1)) && ($vserver_old->{diskinodes_s} > $vserver_new->{diskinodes_h})) {
	    $cce->bye('FAIL', "[[vz-vserver.diskinodes_mismatch]]");
    	    exit(1);
    }
}

$cce->bye('SUCCESS');
exit(0);

