#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: mn_restore.pl, v 1.0.0.3 Sun 23 Dec 2007 09:20:27 PM CET mstauber Exp $
# Copyright 2006-2007 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.
#
# This script is used by "Backup" / "Backup Restore" to do the actual local or remote
# restore of backups. For that purpose it reads the CCE objects Vserver_Restore to find
# out what to restore to which location and from what source (local or remote). 
#


use CCE;
use POSIX qw(isalpha);
use Socket;
use Sys::Hostname;
use POSIX 'strftime';

my $cce = new CCE;

$cce->connectfd();
#$cce->connectuds(); # only for debugging

$oid = $cce->event_oid();
$obj = $cce->event_object();

$host = hostname();
$addr = inet_ntoa(scalar(gethostbyname($host)) || 'localhost');
$now = localtime time;

$restore = $cce->event_object();
$restore_new = $cce->event_new();
$restore_old = $cce->event_old();

# Get the values from CCE and line up our ducks:

$res_originalvpsid 	= $restore_new->{originalvpsid};
$res_ip			= $restore_new->{ip};
$res_hostname		= $restore_new->{hostname};
$res_localrestore 	= $restore_new->{localrestore};
$res_remoterestore 	= $restore_new->{remoterestore};
$res_newvpsid 		= $restore_new->{newvpsid};

# Write to tempfile for debugging:
system("rm -f /tmp/restore.debug");
system("touch /tmp/restore.debug");
system("echo $res_originalvpsid  >> /tmp/restore.debug");
system("echo $res_ip  >> /tmp/restore.debug");
system("echo $res_hostname  >> /tmp/restore.debug");
system("echo $res_localrestore  >> /tmp/restore.debug");
system("echo $res_remoterestore  >> /tmp/restore.debug");
system("echo $res_newvpsid  >> /tmp/restore.debug");

# Validate consistency of our input:
if (($res_originalvpsid ne "") && ($res_ip ne "") && ($res_hostname  ne "") && ($res_localrestore ne "") && ($res_remoterestore ne "") && ($res_newvpsid ne "")) {

	# Do we restore from local or remote?
	if (($res_localrestore == "1") && ($res_remoterestore == "0")) {
		&do_local_restore;
	}
	if (($res_localrestore == "0") && ($res_remoterestore == "1")) {
		&do_remote_restore;
	}
}
else {
	$cce->bye('FAILURE');
	exit(1);
}
$cce->bye('SUCCESS');
exit(0);

#
### Subs:
#

# Local restore:
sub do_local_restore {

        # Do the dirty deeds - if we have what we need, like a backup VPS and a backup conf:
        if ((-d "/vz/backup/vps/$res_originalvpsid" ) && (-e "/vz/backup/conf/$res_originalvpsid.conf" )) {

	    # Stop the target VPS - if it already exists:
	    if (-d "/vz/private/$res_newvpsid") {
            	system("/usr/sbin/vzctl stop $res_newvpsid --fast");
		system("sleep 5");
	    }

            # Copy the data from the backup location to the production environment:
            system("/usr/bin/rsync -q --progress -var /vz/backup/vps/$res_originalvpsid/ /vz/private/$res_newvpsid/ --delete-after");
            system("/bin/cp /vz/backup/conf/$res_originalvpsid.conf /etc/vz/conf/$res_newvpsid.conf");

	    # If we restore onto a new VPSID, we cannot keep the original IP address:
	    if ($res_originalvpsid ne $res_newvpsid) {
		system("/bin/cat /etc/vz/conf/$res_newvpsid.conf | /bin/sed s/$res_ip/127.0.0.$res_newvpsid/ > /etc/vz/conf/$res_newvpsid.conf.bak");
		system("/bin/mv /etc/vz/conf/$res_newvpsid.conf.bak /etc/vz/conf/$res_newvpsid.conf");
	    }
	    system("/bin/chmod 644 /etc/vz/conf/$res_newvpsid.conf");
	
	    if ($res_originalvpsid eq $res_newvpsid) {
                # Start the VPS again:
	        system("/usr/sbin/vzctl start $res_newvpsid");
	    }
        }
        else {
                print STDERR "Error while restoring backup:\n";
                print STDERR "Unable to find /vz/backup/vps/$res_originalvpsid and/or /vz/backup/conf/$res_originalvpsid.conf !\n";
        }
}

# Remote restore:
sub do_remote_restore {

    # First find out if either local or remote backup is enabled - or even both:
    @oids = $cce->find('VserverMasterNode');
    if (!defined($oids[0])) {
	$cce->bye('FAIL',
    	    "[[vz-vserver.No_VserverMasterNode_Object_in_CCE']]");
    	    print STDERR "No_VserverMasterNode_Object_in_CCE";
    	    exit(1);
    }

    # Object VserverMasterNode is present. Get its contends:
    ($ok, $solbackupValues) = $cce->get($oids[0]);
    if ($ok) {

        # Check if Remote Backup details are filled in:
        if (($solbackupValues->{'MN_BackupRemoteTargetHost'} ne "") && ($solbackupValues->{'MN_BackupRemoteLocation'} ne "") &&
	    ($solbackupValues->{'MN_BackupRemoteTargetUser'} ne "")) {

    	    # Remote backup storage location details are filled in.
    	    $MN_Remote_is_present = 1;
    	    $MN_BackupRemoteTargetHost = $solbackupValues->{'MN_BackupRemoteTargetHost'};
    	    $MN_BackupRemoteLocation = $solbackupValues->{'MN_BackupRemoteLocation'};
	    $MN_BackupRemoteTargetPort = $solbackupValues->{'MN_BackupRemoteTargetPort'};
    	    $MN_BackupRemoteTargetUser = $solbackupValues->{'MN_BackupRemoteTargetUser'};
    	}

    	if ((!$MN_BackupRemoteTargetPort) || ($MN_BackupRemoteTargetPort == "")) {
        	$MN_BackupRemoteTargetPort = "22";
    	}

    	# Do the dirty deeds - if we have what we need, like a backup VPS and a backup conf:
    	if (($MN_BackupRemoteTargetHost ne "") || ($MN_BackupRemoteLocation ne "")) {

    	    # Stop the target VPS - if it already exists:
    	    if (-d "/vz/private/$res_newvpsid") {
                system("/usr/sbin/vzctl stop $res_newvpsid --fast");
                system("sleep 5");
    	    }

            # Copy the data from the backup location to the production environment:
#            system("/usr/bin/rsync -q --progress -var -e ssh $MN_BackupRemoteTargetHost:$MN_BackupRemoteLocation/vps/$res_originalvpsid/ /vz/private/$res_newvpsid/ --delete-after");
#            system("/usr/bin/rsync -q --progress -var -e ssh $MN_BackupRemoteTargetHost:$MN_BackupRemoteLocation/conf/$res_originalvpsid.conf /etc/vz/conf/$res_newvpsid.conf --delete-after");
            system("/usr/bin/rsync -q --progress -var -e 'ssh -p $MN_BackupRemoteTargetPort' $MN_BackupRemoteTargetHost:$MN_BackupRemoteLocation/vps/$res_originalvpsid/ /vz/private/$res_newvpsid/ --delete-after");
            system("/usr/bin/rsync -q --progress -var -e 'ssh -p $MN_BackupRemoteTargetPort' $MN_BackupRemoteTargetHost:$MN_BackupRemoteLocation/conf/$res_originalvpsid.conf /etc/vz/conf/$res_newvpsid.conf --delete-after");

	    # If we restore onto a new VPSID, we cannot keep the original IP address:
	    if ($res_originalvpsid ne $res_newvpsid) {
		system("/bin/cat /etc/vz/conf/$res_newvpsid.conf | /bin/sed s/$res_ip/127.0.0.$res_newvpsid/ > /etc/vz/conf/$res_newvpsid.conf.bak");
		system("/bin/mv /etc/vz/conf/$res_newvpsid.conf.bak /etc/vz/conf/$res_newvpsid.conf");
	    }

	    system("/bin/chmod 644 /etc/vz/conf/$res_newvpsid.conf");

	    if ($res_originalvpsid eq $res_newvpsid) {
                # Start the VPS again:
	        system("/usr/sbin/vzctl start $res_newvpsid");
	    }
        }
        else {
                print STDERR "Error while restoring backup:\n";
                print STDERR "No remote host and remote directory specified!\n";
        }
    }
}

####

$cce->bye('SUCCESS');
exit(0);

