#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: mnbackup_info.pl, v 1.0.0.3 Sun 23 Dec 2007 09:47:17 PM CET mstauber Exp $
# Copyright 2006-2007 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.
#
# This script is used by "Backup" / "Backup Restore" to find out which remote and/or
# local backup sets are available for restore. The main purpose is to populate CCE
# with that information - to make it possible to restore VPSs after an OS restore of
# the master node.
#

use CCE;
use POSIX qw(isalpha);
use Socket;
use Sys::Hostname;
use POSIX 'strftime';

my $cce = new CCE;

$cce->connectfd();
#$cce->connectuds(); # only for debugging

$oid = $cce->event_oid();
$obj = $cce->event_object();

$host = hostname();
$addr = inet_ntoa(scalar(gethostbyname($host)) || 'localhost');
$now = localtime time;

# First find out if either local or remote backup is enabled - or even both:
@oids = $cce->find('VserverMasterNode');
if (!defined($oids[0])) {
	$cce->bye('FAIL',
        	"[[vz-vserver.No_VserverMasterNode_Object_in_CCE']]");
	print STDERR "No_VserverMasterNode_Object_in_CCE";
        exit(1);
}
# Object VserverMasterNode is present. Get its contends:
($ok, $solbackupValues) = $cce->get($oids[0]);
if ($ok) {
    # Check if local or remote backup is enabled:
    if (($solbackupValues->{'MN_LocalBackupEnabled'} == "1") || ($solbackupValues->{'MN_RemoteBackupEnabled'} == "1")) {
        $backup_enabled = "1";
    }

    # Check if Remote Backup details are filled in:
    if (($solbackupValues->{'MN_BackupRemoteTargetHost'} ne "") && ($solbackupValues->{'MN_BackupRemoteLocation'} ne "") && 
	($solbackupValues->{'MN_BackupRemoteTargetUser'} ne "")) {
	# Remote backup storage location details are filled in.
	$MN_Remote_is_present = 1;
	$MN_BackupRemoteTargetHost = $solbackupValues->{'MN_BackupRemoteTargetHost'};
	$MN_BackupRemoteLocation = $solbackupValues->{'MN_BackupRemoteLocation'};
        $MN_BackupRemoteTargetPort = $solbackupValues->{'MN_BackupRemoteTargetPort'};
	$MN_BackupRemoteTargetUser = $solbackupValues->{'MN_BackupRemoteTargetUser'};
    }

    if ((!$MN_BackupRemoteTargetPort) || ($MN_BackupRemoteTargetPort == "")) {
        $MN_BackupRemoteTargetPort = "22";
    }

    # Check if local backup is present:
    if (($solbackupValues->{'MN_BackupLocalLocation'} == "") || ($solbackupValues->{'MN_BackupLocalLocation'} == "/")) {
        $local_target_dir = "/vz/backup";
    }
    else {
        $local_target_dir = $solbackupValues->{'MN_BackupLocalLocation'};
    }

    # Check if directories for local backup storage are present on the master node already:
    if ((-d $local_target_dir) && (-d "$local_target_dir/vps") && (-d "$local_target_dir/conf")) {

	# Now check if config files of locally stored VPS's are found:
	opendir DIRH, "$local_target_dir/conf" or die "couldn't open: $!";
	foreach (sort readdir DIRH) {
	    if ($_ =~ /^\d+$\.conf/) {
		$vps_conf = $_;
		@my_vps_conf = split(/.conf/,$vps_conf);
		$locallystoredvps = $my_vps_conf[0];

		# Start parsing the config file and gather the data we need:
    		if (-e "$local_target_dir/conf/$vps_conf") {
		    $num_local_vps = 0;
        	    open(CFILE,"$local_target_dir/conf/$vps_conf") || die("Could not find $local_target_dir/conf/$vps_conf. $!");

        	    while ($line = <CFILE>) {
            		chomp($line);
            		next if $line =~ /^\s*$/; # skip blank lines

			if ($locallystoredvps ne $BACKUPINFO[$locallystoredvps][0]) {
				push(@locally_stored_vpsids, $locallystoredvps);
				$num_local_vps++;
			}

			# VPSID:
			$BACKUPINFO[$locallystoredvps][0] = $locallystoredvps;

			# IP:
			if ($line =~ /^IP_ADDRESS=\"(.*)\"/) {
			    $BACKUPINFO[$locallystoredvps][1] = $1;
		    	}
            		if ($line =~ /^IP_ADDRESS=\"(.*)\s$*/) {
		            @my_ip_list = split(/ /,$1);
                	    $my_ip = $my_ip_list[0];
			    $BACKUPINFO[$locallystoredvps][1] = $my_ip;
            		}

			# Hostname:
            		if ($line =~ /^HOSTNAME=\"(.*)\"/) {
			    $BACKUPINFO[$locallystoredvps][2] = $1;
            		}

			# Last modified:
			my $time = (stat "$local_target_dir/conf/$vps_conf")[9];
			$BACKUPINFO[$locallystoredvps][4] = strftime '%a %b %d %I:%M:%S %Y', localtime $time;

			# Local Backup present:
			if (-d "$local_target_dir/vps/$locallystoredvps") {
			    $BACKUPINFO[$locallystoredvps][3] = "1";
			}
        	    }
    		}
    		close(CFILE);
	    }
	}
	closedir DIRH; 
    }#End for: "Check if directories for local backup storage are present on the master node already"

    # Check if REMOTE backup files are present and parseable:
    if ($MN_Remote_is_present == 1) {

	# Create the local backup directory in case it is missing:
	if (!-d "$local_target_dir") {
		system("/bin/mkdir $local_target_dir");
	}

	# Create the temp directories so that we can pull the config files and parse them:
	if (!-d "$local_target_dir/tmp") {
		system("/bin/mkdir $local_target_dir/tmp");
	}

	# Get the config files from the remote storage server:
#	system("/usr/bin/rsync -q --progress -var -e ssh $MN_BackupRemoteTargetHost:$MN_BackupRemoteLocation/conf $local_target_dir/tmp --delete-after");
	system("/usr/bin/rsync -q --progress -var -e 'ssh -p $MN_BackupRemoteTargetPort' $MN_BackupRemoteTargetHost:$MN_BackupRemoteLocation/conf $local_target_dir/tmp --delete-after");

	# Now parse the retrieved config files:
        opendir DIRH, "$local_target_dir/tmp/conf" or die "couldn't open: $!";
        foreach (sort readdir DIRH) {
            if ($_ =~ /^\d+$\.conf/) {
                $vps_conf = $_;
                @my_vps_conf = split(/.conf/,$vps_conf);
                $remotelystoredvps = $my_vps_conf[0];

                # Start parsing the config file and gather the data we need:
                if (-e "$local_target_dir/tmp/conf/$vps_conf") {
                    $num_remote_vps = 0;
                    open(CFILE,"$local_target_dir/tmp/conf/$vps_conf") || die("Could not find $local_target_dir/tmp/conf/$vps_conf. $!");

                    while ($line = <CFILE>) {
                        chomp($line);
                        next if $line =~ /^\s*$/; # skip blank lines

                        if ($num_remote_vps ne $BACKUPINFO[$remotelystoredvps][0]) {
                                push(@remotely_stored_vpsids, $remotelystoredvps);
                                $num_remote_vps++;
                        }

                        # VPSID:
                        $BACKUPINFO[$remotelystoredvps][0] = $remotelystoredvps;

                        # IP:

                        if ($line =~ /^IP_ADDRESS=\"(.*)\"/) {
                            $BACKUPINFO[$remotelystoredvps][1] = $1;
                        }
                        if ($line =~ /^IP_ADDRESS=\"(.*)\s$*/) {
                            @my_ip_list = split(/ /,$1);
                            $my_ip = $my_ip_list[0];
                            $BACKUPINFO[$remotelystoredvps][1] = $my_ip;
                        }

                        if ($line =~ /^IP_ADDRESS=\"(.*)\s\"/) {
                                print "1:" . $line . "\n";
                                if ($line =~ /^IP_ADDRESS=\"(.*)\"/) {
                                        print "2:" . $line . "\n";
                                }
                            $BACKUPINFO[$locallystoredvps][1] = $1;
                        }

                        # Hostname:
                        if ($line =~ /^HOSTNAME=\"(.*)\"/) {
                            $BACKUPINFO[$remotelystoredvps][2] = $1;
                        }

                        # Last modified:
                        $time = (stat "$local_target_dir/tmp/conf/$vps_conf")[9];
                        $BACKUPINFO[$remotelystoredvps][6] = strftime '%a %b %d %I:%M:%S %Y', localtime $time;

			# Remote Backup present:
                        $BACKUPINFO[$remotelystoredvps][5] = "1";
                    }
                }
                close(CFILE);
            }
        }
        closedir DIRH;
	# Remove temp directory
        if (-d "$local_target_dir/tmp") {
                system("/bin/rm -R $local_target_dir/tmp");
        }
    } # End for: "Check if REMOTE backup files are present and parseable:"   
}

# Merge the arrays @locally_stored_vpsids and @remotely_stored_vpsids:
@remote_and_local_vpsids = (@locally_stored_vpsids, @remotely_stored_vpsids);

# Now make sure we get only the unique VPSID's that were stored in both arrays:
undef %saw;
@saw{@remote_and_local_vpsids} = ();
@out = sort keys %saw;  # remove sort if undesired

# For Debugging purpose only:
#foreach $vpsid (@out) {
	#print "--------------------------------------\n";
	#print "VPSID: 		$BACKUPINFO[$vpsid][0] \n";
	#print "IP_ADDRESS: 	$BACKUPINFO[$vpsid][1] \n";
	#print "HOSTNAME: 	$BACKUPINFO[$vpsid][2] \n";
	#print "LBpresent: 	$BACKUPINFO[$vpsid][3] \n";
	#print "LBtimestamp: 	$BACKUPINFO[$vpsid][4] \n";
	#print "LRpresent: 	$BACKUPINFO[$vpsid][5] \n";
	#print "LRtimestamp: 	$BACKUPINFO[$vpsid][6] \n";
#}

# Now the tricky part:
#
# Usually the CCE objects 'Vserver_Backup' are present and contain the information
# if a VPS has local and/or remote backup enabled. It also stores when the last time
# was that such a backup has been run.
#
# Now IF someone just restored the master node from the ISO, he may not have that info
# in CCE yet. So this script needs to determine if the objects 'Vserver_Backup' are 
# present. 
#
# If they ARE there, we do nothing.
#
# If they ARE NOT there, we recreate them from scratch with the info we just gathered.

# Check CCE if 'Vserver_Backup' objects are present for the gathered VPSIDs:

@ccebackupvpsids = $cce->findx('Vserver_Backup', '');
$total = scalar(@ccebackupvpsids);
if ($total > 0) {
    # Ok, we found something.
    # This means we need to update instead of create:
    foreach $vpsid (@out) {
        @sys_oid = $cce->find('Vserver_Backup', {'vpsid' => $vpsid});
        if ($#sys_oid ne -1) {
           ($ok) = $cce->set($sys_oid[0], '',{
            'vpsid' => $BACKUPINFO[$vpsid][0],
            'ip' => $BACKUPINFO[$vpsid][1],
            'hostname' => $BACKUPINFO[$vpsid][2],
            'LBpresent' => $BACKUPINFO[$vpsid][3],
            'LBtimestamp' => $BACKUPINFO[$vpsid][4],
            'LRpresent' => $BACKUPINFO[$vpsid][5],
            'LRtimestamp' => $BACKUPINFO[$vpsid][6],
            'timestamp' => time()
            });
        }
    }
}
else {
    # Did not find any CCE Objects with the name Vserver_Backup. 
    # So create them now and here:
    foreach $vpsid (@out) {
	@oids = $cce->find('Vserver_Backup', {'vpsid' => $vpsid});
    	if ($#oids < 0) {
            ($ok) = $cce->create('Vserver_Backup', {
            'vpsid' => $BACKUPINFO[$vpsid][0],
            'ip' => $BACKUPINFO[$vpsid][1],
            'hostname' => $BACKUPINFO[$vpsid][2],
            'LBpresent' => $BACKUPINFO[$vpsid][3],
            'LBtimestamp' => $BACKUPINFO[$vpsid][4],
            'LRpresent' => $BACKUPINFO[$vpsid][5],
            'LRtimestamp' => $BACKUPINFO[$vpsid][6],
            'timestamp' => time()
            });
	}
    }
}

$cce->bye('SUCCESS');
exit(0);

####

# Legend:
#
# $BACKUPINFO[$VPSID][0] = VPSID
# $BACKUPINFO[$VPSID][1] = IP_ADDRESS
# $BACKUPINFO[$VPSID][2] = HOSTNAME
# $BACKUPINFO[$VPSID][3] = LBpresent
# $BACKUPINFO[$VPSID][4] = LBtimestamp
# $BACKUPINFO[$VPSID][5] = LRpresent
# $BACKUPINFO[$VPSID][6] = LRtimestamp
# $BACKUPINFO[$VPSID][7] = VserverBackupLocalEnabled (not used here)
# $BACKUPINFO[$VPSID][8] = VserverBackupRemoteEnabled (not used here)
# $BACKUPINFO[$VPSID][9] = VserverBackupRemoteAuthority (not used here)



