#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: update_conf.pl,v 1.0.0.0 Thu 16 Nov 2006 03:16:57 AM CET mstauber Exp $
# Copyright 2006 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.
#
# Handler which writes / updates VZ config files.
#

use CCE;
use POSIX qw(isalpha);

my $DEBUG = 0;

my $cce = new CCE('Domain' => 'vz-vserver');

if ($DEBUG == 0) {
    $cce->connectfd();
}
else {
    $cce->connectuds(); # only for debugging
}

$vzconf_path = '/etc/sysconfig/vz-scripts';

$vserver = $cce->event_object();
$vserver_new = $cce->event_new();
$vserver_old = $cce->event_old();

if ($DEBUG == 1) {
    system("date > /tmp/update_conf");
}

# Do we use the GUI submitted individual values, or do we switch to those
# from a pre-defined ORIGIN_SAMPLE which was selected in the GUI?
if (($vserver_new->{conf_action} eq "changeconfig") && ($vserver_new->{ORIGIN_SAMPLE})) {
    $need_change_config = 1;
    system("echo \"dirty: $vserver_new->{conf_action}\" >> /tmp/update_conf");
    system("echo \"Sample: $vserver_new->{ORIGIN_SAMPLE}\" >> /tmp/update_conf");
}

@oids= $cce->find("Vserver");
($ok, $system) = $cce->get($oids[0]);

if ($vserver->{vpsid}) {
    $vpsid = $vserver->{vpsid};
    if ($DEBUG == 1) {
	system("echo \"VPSID: $vpsid\" >> /tmp/update_conf");

    }
}
else {
    $cce->bye('FAIL', "[[vz-vserver.unableToDetermineVPSID]]");
    exit(1);
}

##
# IP address allocation stuff:
##

# Find the relevant objects through their vpsid in vpsIPalloc objects:
@oids = $cce->find('vpsIPalloc', {'vpsid' => $vpsid});
if ($#oids < 0) {
    if ($DEBUG == 1) {
	system("echo 'Unable to determine VPSID based on vpsIPalloc' >> /tmp/update_conf");
    }
    # We may not have any extra IPs allocated for the VPS.
    # So we DO NOT error out here:
    #$cce->bye('FAIL', "[[vz-vserver.unableToDetermineVPSID]]");
    #exit(1);
}
else {
    # VPSID found in CCE.
    foreach $line (@oids) {
            ($ok, $system) = $cce->get($line);
            $func = push(@ips, $system->{min});
    }
    # How many IP's we got?
    $num = $#ips;
    # Sort the IPs:
    @sorted_ips = sort { $a cmp $b } @ips;
    # Line them up in the single string $iplist, separated by spaces:
    foreach $value (@sorted_ips) {
        if ($num > 0) {
            $iplist .= $value . ";";
            $num--;
        }
        else {
            $iplist .= $value;
        }
    }
}


# Find the relevant object through its vpsid in Vserver:
@oids = $cce->find('Vserver', {'vpsid' => $vpsid});
if ($#oids < 0) {
    $cce->bye('FAIL', "[[vz-vserver.unableToDetermineVPSID]]");
    if ($DEBUG == 1) {
	system("echo 'VPSID not found in CCE.' >> /tmp/update_conf");
    }
    exit(1);
}
else {
    # get the system object
    ($sys_oid) = $cce->find('Vserver', {'vpsid' => $vpsid});
    ($ok, $vpsdata) = $cce->get($sys_oid);

    if ($ok) {
	if ($DEBUG == 1) {
	    system("echo 'VPSDATA found in CCE.' >> /tmp/update_conf");
	}
	if ($need_change_config == 0) {
	    # Use GUI submitted new individual values:
	    &use_gui_input;
	}
	else {
	    # use a canned ORIGIN_SAMPLE instead:
	    &use_canned_config;
	}
    }
    else {
	if ($DEBUG == 1) {
	    system("echo 'VPSDATA retrieval errored out.' >> /tmp/update_conf");
	}
    }
}

# Use GUI submitted new individual values:
sub use_gui_input {

    if ($DEBUG == 1) {
        system("echo 'Using GUI submitted values' >> /tmp/update_conf");
    }
    
	vps_write_conf(
    	    $vpsdata->{'vpsid'},
    	    $vpsdata->{'ip'},
    	    $vpsdata->{'hostname'},
    	    $vpsdata->{'NAMESERVER'},
    	    $vpsdata->{'ONBOOT'},
    	    $vpsdata->{'cpuunits'},
    	    $vpsdata->{'avnumproc_b'},
    	    $vpsdata->{'avnumproc_l'},
    	    $vpsdata->{'numproc_b'},
    	    $vpsdata->{'numproc_l'},
    	    $vpsdata->{'numtcpsock_b'},
    	    $vpsdata->{'numtcpsock_l'},
    	    $vpsdata->{'numothersock_b'},
    	    $vpsdata->{'numothersock_l'},
    	    $vpsdata->{'OSTEMPLATE'},
    	    $vpsdata->{'ORIGIN_SAMPLE'},
    	    $vpsdata->{'vmguarpages_b'},
    	    $vpsdata->{'vmguarpages_l'},
    	    $vpsdata->{'kmemsize_b'},
    	    $vpsdata->{'kmemsize_l'},
    	    $vpsdata->{'tcpsndbuf_b'},
    	    $vpsdata->{'tcpsndbuf_l'},
    	    $vpsdata->{'tcprcvbuf_b'},
    	    $vpsdata->{'tcprcvbuf_l'},
    	    $vpsdata->{'othersockbuf_b'},
    	    $vpsdata->{'othersockbuf_l'},
    	    $vpsdata->{'dgramrcvbuf_b'},
    	    $vpsdata->{'dgramrcvbuf_l'},
    	    $vpsdata->{'oomguarpages_b'},
    	    $vpsdata->{'oomguarpages_l'},
    	    $vpsdata->{'lockedpages_b'},
    	    $vpsdata->{'lockedpages_l'},
    	    $vpsdata->{'shmpages_b'},
    	    $vpsdata->{'shmpages_l'},
    	    $vpsdata->{'privvmpages_b'},
    	    $vpsdata->{'privvmpages_l'},
    	    $vpsdata->{'numfile_b'},
    	    $vpsdata->{'numfile_l'},
    	    $vpsdata->{'numflock_b'},
    	    $vpsdata->{'numflock_l'},
    	    $vpsdata->{'numpty_b'},
    	    $vpsdata->{'numpty_l'},
    	    $vpsdata->{'numsiginfo_b'},
    	    $vpsdata->{'numsiginfo_l'},
    	    $vpsdata->{'dcachesize_b'},
    	    $vpsdata->{'dcachesize_l'},
    	    $vpsdata->{'physpages_b'},
    	    $vpsdata->{'physpages_l'},
    	    $vpsdata->{'numiptent_b'},
    	    $vpsdata->{'numiptent_l'},
    	    $vpsdata->{'diskspace_s'},
    	    $vpsdata->{'diskspace_h'},
    	    $vpsdata->{'diskinodes_s'},
    	    $vpsdata->{'diskinodes_h'},
    	    $vpsdata->{'QUOTATIME'},
    	    $vpsdata->{'DISK_QUOTA'},
    	    $vpsdata->{'QUOTAUGIDLIMIT'},
	    $iplist
        );
}

sub vps_write_conf {

        my $vpsid                 	= shift;
        my $ip             		= shift;
        my $hostname                   	= shift;
        my $NAMESERVER                 	= shift;
        my $ONBOOT                 	= shift;
        my $cpuunits                	= shift;
	my $avnumproc_b			= shift;
	my $avnumproc_l			= shift;
	my $numproc_b			= shift;
	my $numproc_l			= shift;
	my $numtcpsock_b		= shift;
	my $numtcpsock_l		= shift;
	my $numothersock_b		= shift;
	my $numothersock_l		= shift;
	my $OSTEMPLATE			= shift;
	my $ORIGIN_SAMPLE		= shift;
	my $vmguarpages_b		= shift;
	my $vmguarpages_l		= shift;
	my $kmemsize_b			= shift;
	my $kmemsize_l			= shift;
	my $tcpsndbuf_b			= shift;
	my $tcpsndbuf_l			= shift;
	my $tcprcvbuf_b			= shift;
	my $tcprcvbuf_l			= shift;
	my $othersockbuf_b		= shift;
	my $othersockbuf_l		= shift;
	my $dgramrcvbuf_b		= shift;
	my $dgramrcvbuf_l		= shift;
	my $oomguarpages_b		= shift;
	my $oomguarpages_l		= shift;
	my $lockedpages_b		= shift;
	my $lockedpages_l		= shift;
	my $shmpages_b			= shift;
	my $shmpages_l			= shift;
	my $privvmpages_b		= shift;
	my $privvmpages_l		= shift;
	my $numfile_b			= shift;
	my $numfile_l			= shift;
	my $numflock_b			= shift;
	my $numflock_l			= shift;
	my $numpty_b			= shift;
	my $numpty_l			= shift;
	my $numsiginfo_b		= shift;
	my $numsiginfo_l		= shift;
	my $dcachesize_b		= shift;
	my $dcachesize_l		= shift;
	my $physpages_b			= shift;
	my $physpages_l			= shift;
	my $numiptent_b			= shift;
	my $numiptent_l			= shift;
	my $diskspace_s			= shift;
	my $diskspace_h			= shift;
	my $diskinodes_s		= shift;
	my $diskinodes_h		= shift;
	my $QUOTATIME			= shift;
	my $DISK_QUOTA			= shift;
    	my $QUOTAUGIDLIMIT		= shift;
	my $iplist			= shift;

	if (($DISK_QUOTA eq "Yes") || ($DISK_QUOTA eq "yes")) {
	    $DISK_QUOTA = "yes";
	}
	else {	
	    $DISK_QUOTA = "no";
	}	

	if (($ONBOOT eq "Yes") || ($ONBOOT eq "yes")) {
	    $ONBOOT = "yes";
	}
	else {	
	    $ONBOOT = "no";
	}	

	# De-Scalarize $NAMESERVER:
	@dns = split('&', $NAMESERVER);
	shift(@dns);
	$nameservers = join(" ", @dns);

        open(CONF, ">$vzconf_path/$vpsid.conf") || die "Can't write to VPS config file $vzconf_path/$vpsid.conf!";
        print CONF "
#  Copyright (C) 2000-2006 SWsoft. All rights reserved.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

ONBOOT=\"$ONBOOT\"

# UBC parameters (in form of barrier:limit)
# Primary parameters
AVNUMPROC=\"$avnumproc_b:$avnumproc_l\"
NUMPROC=\"$numproc_b:$numproc_l\"
NUMTCPSOCK=\"$numtcpsock_b:$numtcpsock_l\"
NUMOTHERSOCK=\"$numothersock_b:$numothersock_l\"
VMGUARPAGES=\"$vmguarpages_b:$vmguarpages_l\"
# Secondary parameters
KMEMSIZE=\"$kmemsize_b:$kmemsize_l\"
TCPSNDBUF=\"$tcpsndbuf_b:$tcpsndbuf_l\"
TCPRCVBUF=\"$tcprcvbuf_b:$tcprcvbuf_l\"
OTHERSOCKBUF=\"$othersockbuf_b:$othersockbuf_l\"
DGRAMRCVBUF=\"$dgramrcvbuf_b:$dgramrcvbuf_l\"
OOMGUARPAGES=\"$oomguarpages_b:$oomguarpages_l\"
# Auxiliary parameters
LOCKEDPAGES=\"$lockedpages_b:$lockedpages_l\"
SHMPAGES=\"$shmpages_b:$shmpages_l\"
PRIVVMPAGES=\"$privvmpages_b:$privvmpages_l\"
NUMFILE=\"$numfile_b:$numfile_l\"
NUMFLOCK=\"$numflock_b:$numflock_l\"
NUMPTY=\"$numpty_b:$numpty_l\"
NUMSIGINFO=\"$numsiginfo_b:$numsiginfo_l\"
DCACHESIZE=\"$dcachesize_b:$dcachesize_l\"

PHYSPAGES=\"$physpages_b:$physpages_l\"
NUMIPTENT=\"$numiptent_b:$numiptent_l\"

# Disk quota parameters (in form of softlimit:hardlimit)
DISKSPACE=\"$diskspace_s:$diskspace_h\"
DISKINODES=\"$diskinodes_s:$diskinodes_h\"
QUOTATIME=\"$QUOTATIME\"
DISK_QUOTA=$DISK_QUOTA
QUOTAUGIDLIMIT=$QUOTAUGIDLIMIT

# CPU fair sheduler parameter
CPUUNITS=\"$cpuunits\"
VE_ROOT=\"/vz/root/\$VEID\"
VE_PRIVATE=\"/vz/private/\$VEID\"
OSTEMPLATE=\"$OSTEMPLATE\"
ORIGIN_SAMPLE=\"$ORIGIN_SAMPLE\"
IP_ADDRESS=\"$ip\"
HOSTNAME=\"$hostname\"
NAMESERVER=\"$nameservers\"
";
close CONF;

	# Apply changes to VPS (regardless if it is running or stopped) and rewrite config: 
	system("/usr/sbin/vzctl set $vpsid --ipdel all --ipadd $ip --hostname $hostname --nameserver \"$nameservers\" --cpuunits $cpuunits --avnumproc $avnumproc_b:$avnumproc_l --numproc $numproc_b:$numproc_l --numtcpsock $numothersock_b:$numothersock_l --numothersock $numothersock_b:$numothersock_l --vmguarpages $vmguarpages_b:$vmguarpages_l --kmemsize $kmemsize_b:$kmemsize_l --tcpsndbuf $tcprcvbuf_b:$tcprcvbuf_l --tcprcvbuf $tcprcvbuf_b:$tcprcvbuf_l --othersockbuf $othersockbuf_b:$othersockbuf_l --dgramrcvbuf $dgramrcvbuf_b:$dgramrcvbuf_l --oomguarpages $oomguarpages_b:$oomguarpages_l --lockedpages $lockedpages_b:$lockedpages_l --shmpages $shmpages_b:$shmpages_l --privvmpages $privvmpages_b:$privvmpages_l --numfile $numfile_b:$numfile_l --numflock $numflock_b:$numflock_l --numpty $numpty_b:$numpty_l --numsiginfo $numsiginfo_b:$numsiginfo_l --dcachesize $dcachesize_b:$dcachesize_l --physpages $physpages_b:$physpages_l --numiptent $numiptent_b:$numiptent_l --diskspace $diskspace_s:$diskspace_h --diskinodes $diskinodes_s:$diskinodes_h --quotatime $QUOTATIME --quotaugidlimit $QUOTAUGIDLIMIT --save");
	
	# Add extra IP's - have to do it one by one <sigh>:
	@values = split(';', $iplist);
  	foreach $ip_from_list (@values) {
		system("/usr/sbin/vzctl set $vpsid --ipadd $ip_from_list --save");
  	}
	# Fix permissions:
	system("/bin/chmod 644 $vzconf_path/$vpsid.conf");

	&validate_conf;
}

sub use_canned_config {

    # Open canned config file and parse the info:
    $canned_config_file = $vzconf_path . '/ve-' . $vserver_new->{ORIGIN_SAMPLE} . ".conf-sample";

    if ($DEBUG == 1) {
        system("echo 'Using canned config file $canned_config_file' >> /tmp/update_conf");
    }

    # We NEED to read one config file which we know has all values: (uhm ... this doesn't really work, hmkay?)
    if (-e "/etc/vz/conf/ve-aventurine-server.conf-sample") {
        open(CPUDUMP,"/etc/vz/conf/ve-aventurine-server.conf-sample") || die("Could not find '/etc/vz/conf/ve-aventurine-server.conf-sample'. $!");
	&parse_the_config_file;
        close(CPUDUMP);
    }
    # These values may or may have not been changed in the GUI.
    # So if the user did not change/update them in the GUI,
    # then we used the last known values instead:
    if ((!$vserver_new->{ip}) && ($vserver_old->{ip})) {
	$IP = $vserver_old->{ip};
    }
    else {
	$IP = $vserver_new->{ip};
    }
    if ((!$vserver_new->{hostname}) && ($vserver_old->{hostname})) {
	$HOSTNAME = $vserver_old->{hostname};
    }
    else {
	$HOSTNAME = $vserver_new->{hostname};
    }
    if ((!$vserver_new->{NAMESERVER}) && ($vserver_old->{NAMESERVER})) {
	$NAMESERVER = $vserver_old->{NAMESERVER};
    }
    else {
	$NAMESERVER = $vserver_new->{NAMESERVER};
    }
    if ((!$vserver_new->{OSTEMPLATE}) && ($vserver_old->{OSTEMPLATE})) {
	$OSTEMPLATE = $vserver_old->{OSTEMPLATE};
    }
    else {
	$OSTEMPLATE = $vserver_new->{OSTEMPLATE};
    }
    
    # Now we parse the config file that the user selected in the GUI.
    # Any value that the user selected config file does NOT contain
    # will be kept from the good sample we have read above.
    if (-e $canned_config_file) {
        open(CPUDUMP,"$canned_config_file") || die("Could not find $canned_config_file. $!");
	&parse_the_config_file;
        close(CPUDUMP);

	# Write new config file:    
        open(CONF, ">$vzconf_path/$vpsid.conf") || die "Can't write to VPS config file $vzconf_path/$vpsid.conf!";
        print CONF "

#  Copyright (C) 2000-2006 SWsoft. All rights reserved.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

ONBOOT=\"$ONBOOT\"

# UBC parameters (in form of barrier:limit)
# Primary parameters
AVNUMPROC=\"$avnumproc_b:$avnumproc_l\"
NUMPROC=\"$numproc_b:$numproc_l\"
NUMTCPSOCK=\"$numtcpsock_b:$numtcpsock_l\"
NUMOTHERSOCK=\"$numothersock_b:$numothersock_l\"
VMGUARPAGES=\"$vmguarpages_b:$vmguarpages_l\"
# Secondary parameters
KMEMSIZE=\"$kmemsize_b:$kmemsize_l\"
TCPSNDBUF=\"$tcpsndbuf_b:$tcpsndbuf_l\"
TCPRCVBUF=\"$tcprcvbuf_b:$tcprcvbuf_l\"
OTHERSOCKBUF=\"$othersockbuf_b:$othersockbuf_l\"
DGRAMRCVBUF=\"$dgramrcvbuf_b:$dgramrcvbuf_l\"
OOMGUARPAGES=\"$oomguarpages_b:$oomguarpages_l\"
# Auxiliary parameters
LOCKEDPAGES=\"$lockedpages_b:$lockedpages_l\"
SHMPAGES=\"$shmpages_b:$shmpages_l\"
PRIVVMPAGES=\"$privvmpages_b:$privvmpages_l\"
NUMFILE=\"$numfile_b:$numfile_l\"
NUMFLOCK=\"$numflock_b:$numflock_l\"
NUMPTY=\"$numpty_b:$numpty_l\"
NUMSIGINFO=\"$numsiginfo_b:$numsiginfo_l\"
DCACHESIZE=\"$dcachesize_b:$dcachesize_l\"

PHYSPAGES=\"$physpages_b:$physpages_l\"
NUMIPTENT=\"$numiptent_b:$numiptent_l\"

# Disk quota parameters (in form of softlimit:hardlimit)
DISKSPACE=\"$diskspace_s:$diskspace_h\"
DISKINODES=\"$diskinodes_s:$diskinodes_h\"
QUOTATIME=\"$QUOTATIME\"
DISK_QUOTA=$DISK_QUOTA
QUOTAUGIDLIMIT=$QUOTAUGIDLIMIT

# CPU fair sheduler parameter
CPUUNITS=\"$cpuunits\"
VE_ROOT=\"/vz/root/\$VEID\"
VE_PRIVATE=\"/vz/private/\$VEID\"
OSTEMPLATE=\"$OSTEMPLATE\"
ORIGIN_SAMPLE=\"$vserver_new->{ORIGIN_SAMPLE}\"
IP_ADDRESS=\"$IP\"
HOSTNAME=\"$HOSTNAME\"
NAMESERVER=\"$NAMESERVER\"
";
close CONF;

	# Apply changes to VPS (regardless if it is running or stopped) and rewrite config: 
	system("/usr/sbin/vzctl set $vpsid --ipdel all --ipadd $IP --hostname $hostname --nameserver $NAMESERVER --cpuunits $cpuunits --avnumproc $avnumproc_b:$avnumproc_l --numproc $numproc_b:$numproc_l --numtcpsock $numothersock_b:$numothersock_l --numothersock $numothersock_b:$numothersock_l --vmguarpages $vmguarpages_b:$vmguarpages_l --kmemsize $kmemsize_b:$kmemsize_l --tcpsndbuf $tcprcvbuf_b:$tcprcvbuf_l --tcprcvbuf $tcprcvbuf_b:$tcprcvbuf_l --othersockbuf $othersockbuf_b:$othersockbuf_l --dgramrcvbuf $dgramrcvbuf_b:$dgramrcvbuf_l --oomguarpages $oomguarpages_b:$oomguarpages_l --lockedpages $lockedpages_b:$lockedpages_l --shmpages $shmpages_b:$shmpages_l --privvmpages $privvmpages_b:$privvmpages_l --numfile $numfile_b:$numfile_l --numflock $numflock_b:$numflock_l --numpty $numpty_b:$numpty_l --numsiginfo $numsiginfo_b:$numsiginfo_l --dcachesize $dcachesize_b:$dcachesize_l --physpages $physpages_b:$physpages_l --numiptent $numiptent_b:$numiptent_l --diskspace $diskspace_s:$diskspace_h --diskinodes $diskinodes_s:$diskinodes_h --quotatime $QUOTATIME --quotaugidlimit $QUOTAUGIDLIMIT --save");
	
	# Add extra IP's - have to do it one by one <sigh>:
	@values = split(';', $iplist);
  	foreach $ip_from_list (@values) {
		system("/usr/sbin/vzctl set $vpsid --ipadd $ip_from_list --save");
  	}
	system("/usr/sbin/vzctl restart $vpsid");

	# Fix permissions:
	system("/bin/chmod 644 $vzconf_path/$vpsid.conf");

	&validate_conf;

	# Now that the config has changed, we cannot simply return to versverModify.php
	# without updating the ressource information of the VPS in CCE. For that purpose
	# we rely on the existing beancounter.pl handler, which we call this way:
#	($sys_oid) = $cce->find('VserverMasterNode');
#    	($ok, $sys) = $cce->get($sys_oid);
#    	($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
#    	    'gui_list_id' => $vpsid,
#	    'gui_list_single' => time()
#    	    });
    }
}

sub parse_the_config_file {

        while ($line = <CPUDUMP>) {
            chomp($line);
            next if $line =~ /^\s*$/; # skip blank lines
            if ($line =~ /^ONBOOT=\"([A-Za-z]\w*)\"/) {
                $onboot = $1;
                if (($onboot eq 'yes') || ($onboot eq 'Yes')) {
                    $ONBOOT = 'yes';
                }
                else {
                    $ONBOOT = 'no';
                }
            }
            if ($line =~ /^AVNUMPROC=\"(.*):(.*)\"/) {
                $avnumproc_b = $1;
                $avnumproc_l = $2;
            }
            if ($line =~ /^NUMPROC=\"(.*):(.*)\"/) {
                $numproc_b = $1;
                $numproc_l = $2;
            }
            if ($line =~ /^NUMTCPSOCK=\"(.*):(.*)\"/) {
                $numtcpsock_b = $1;
                $numtcpsock_l = $2;
            }
            if ($line =~ /^NUMOTHERSOCK=\"(.*):(.*)\"/) {
                $numothersock_b = $1;
                $numothersock_l = $2;
            }
            if ($line =~ /^VMGUARPAGES=\"(.*):(.*)\"/) {
                $vmguarpages_b = $1;
                $vmguarpages_l = $2;
            }
            if ($line =~ /^KMEMSIZE=\"(.*):(.*)\"/) {
                $kmemsize_b = $1;
                $kmemsize_l = $2;
            }
            if ($line =~ /^TCPSNDBUF=\"(.*):(.*)\"/) {
                $tcpsndbuf_b = $1;
                $tcpsndbuf_l = $2;
            }
            if ($line =~ /^TCPRCVBUF=\"(.*):(.*)\"/) {
                $tcprcvbuf_b = $1;
                $tcprcvbuf_l = $2;
            }
            if ($line =~ /^OTHERSOCKBUF=\"(.*):(.*)\"/) {
                $othersockbuf_b = $1;
                $othersockbuf_l = $2;
            }
            if ($line =~ /^DGRAMRCVBUF=\"(.*):(.*)\"/) {
                $dgramrcvbuf_b = $1;
                $dgramrcvbuf_l = $2;
            }
            if ($line =~ /^OOMGUARPAGES=\"(.*):(.*)\"/) {
                $oomguarpages_b = $1;
                $oomguarpages_l = $2;
            }
            if ($line =~ /^LOCKEDPAGES=\"(.*):(.*)\"/) {
                $lockedpages_b = $1;
                $lockedpages_l = $2;
            }
            if ($line =~ /^SHMPAGES=\"(.*):(.*)\"/) {
                $shmpages_b = $1;
                $shmpages_l = $2;
            }
            if ($line =~ /^PRIVVMPAGES=\"(.*):(.*)\"/) {
                $privvmpages_b = $1;
                $privvmpages_l = $2;
            }
            if ($line =~ /^NUMFILE=\"(.*):(.*)\"/) {
                $numfile_b = $1;
                $numfile_l = $2;
            }
            if ($line =~ /^NUMFLOCK=\"(.*):(.*)\"/) {
                $numflock_b = $1;
                $numflock_l = $2;
            }
            if ($line =~ /^NUMPTY=\"(.*):(.*)\"/) {
                $numpty_b = $1;
                $numpty_l = $2;
            }
            if ($line =~ /^NUMSIGINFO=\"(.*):(.*)\"/) {
                $numsiginfo_b = $1;
                $numsiginfo_l = $2;
            }
            if ($line =~ /^DCACHESIZE=\"(.*):(.*)\"/) {
                $dcachesize_b = $1;
                $dcachesize_l = $2;
            }
            if ($line =~ /^PHYSPAGES=\"(.*):(.*)\"/) {
                $physpages_b = $1;
                $physpages_l = $2;
            }
            if ($line =~ /^NUMIPTENT=\"(.*):(.*)\"/) {
                $numiptent_b = $1;
                $numiptent_l = $2;
            }

            if ($line =~ /^DISKSPACE=\"(.*):(.*)\"/) {
                $diskspace_s = $1;
                $diskspace_h = $2;
            }
            if ($line =~ /^DISKINODES=\"(.*):(.*)\"/) {
                $diskinodes_s = $1;
                $diskinodes_h = $2;
            }
            if ($line =~ /^QUOTATIME=\"(.*)\"/) {
                $QUOTATIME = $1;
            }
            $quota = '';
            if ($line =~ /^DISK_QUOTA=\"(.*)\"/) {
                $quota = $1;
                if (($quota eq 'yes') || ($quota eq 'Yes')) {
                    $DISK_QUOTA = 'yes';
                }
                else {
                    $DISK_QUOTA = 'no';
                }
            }
            if (($line =~ /^DISK_QUOTA=(.*)/) && ($quota eq '')){
                $quota = $1;
                if (($quota eq 'yes') || ($quota eq 'Yes')) {
                    $DISK_QUOTA = 'yes';
                }
                else {
                    $DISK_QUOTA = 'yo';
                }
            }
            if ($line =~ /^QUOTAUGIDLIMIT=\"(.*)\"/) {
                $QUOTAUGIDLIMIT = $1;
            }
            if ($line =~ /^CPUUNITS=\"(.*)\"/) {
                $cpuunits = $1;
            }
        }
}

sub validate_conf {
    system("/usr/sbin/vzcfgvalidate /etc/sysconfig/vz-scripts/$vpsid.conf 1>/dev/null 2>/tmp/$vpsid.validate"); 
    if (-e "/tmp/$vpsid.validate") {
        open(CFGVALIDATE,"/tmp/$vpsid.validate");
        while ($line = <CFGVALIDATE>) {
            chomp($line);
            next if $line =~ /^\s*$/; # skip blank lines
	    $cfgvalidate = join "|", $cfgvalidate, $line, "";
	}
	close(CFGVALIDATE);
    }
    system("rm -f /tmp/$vpsid.validate");
    # Push results into CCE - we still have the OID stored in $oids[0], by the way:
    ($ok) = $cce->set($oids[0], '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
        'vpsid' => $vpsid,
        'confValidateResult' => $cfgvalidate
    });
}

# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);


