#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: vz_vserver_backup.pl, v 1.0.0.6 Wed 12 Mar 2008 09:41:55 PM CET mstauber Exp $
# Copyright 2006-2008 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.
#
# This script performs local and remote backing up of virtual private servers. 
# Only VPS's which have either local or remote backup enabled in the GUI will be backed up!
#

use CCE;
use POSIX qw(isalpha);
use Socket;
use Sys::Hostname;
use File::Spec;

my $cce = new CCE;
$cce->connectuds();

$oid = $cce->event_oid();
$obj = $cce->event_object();

$host = hostname();
$addr = inet_ntoa(scalar(gethostbyname($host)) || 'localhost');
$now = localtime time;

print STDERR "=================================================================\n";
print STDERR "Aventurin{e} Backup on $host ($addr) \n";
print STDERR "Starting up at: $now\n";
print STDERR "=================================================================\n\n";

# Remove stale lockfiles if it is older than 120 minutes:
if (-f "/tmp/vz_vserver_backup.lock") {
        system("find /tmp/vz_vserver_backup.lock -type f -cmin +120 -print | xargs rm >/dev/null 2>&1");
}

if (-f "/tmp/vz_vserver_backup.lock") {
        print STDERR "ERROR: Found lockfile /tmp/vz_vserver_backup.lock!\n";
        print STDERR "Please check that no other instance of this program runs!\n\n";
        $now = localtime time;
        print STDERR "=================================================================\n";
        print STDERR "Aventurin{e} Backup on $host ($addr) \n";
        print STDERR "Backup aborted at: $now\n";
        print STDERR "=================================================================\n\n";
        exit 1;
}

# Create lockfile:
system("/bin/touch /tmp/vz_vserver_backup.lock");

# First find out if either local or remote backup is enabled - or even both:
@oids = $cce->find('VserverMasterNode');
if (!defined($oids[0])) {
        print STDERR "VZBACKUP: Sorry, no VserverMasterNode object in CCE found!\n";
	system("/bin/rm -f /tmp/vz_vserver_backup.lock");
        exit 1;
}

# Next find out if there is anything to back up (we may be cluster secondary after all!)
if (!-d "/vz/private") {
        print STDERR "INFO: This node is currently secondary in a cluster.\n";
        print STDERR "So there is nothing to backup on this node.\n\n";
        $now = localtime time;
        print STDERR "=================================================================\n";
        print STDERR "Aventurin{e} Backup on $host ($addr) \n";
        print STDERR "Backup aborted at: $now\n";
        print STDERR "=================================================================\n\n";
	system("/bin/rm -f /tmp/vz_vserver_backup.lock");
        exit 1;
}

($ok, $solbackupValues) = $cce->get($oids[0]);
if ($ok) {
    if (($solbackupValues->{'MN_LocalBackupEnabled'} == "1") || ($solbackupValues->{'MN_RemoteBackupEnabled'} == "1")) {
        $backup_enabled = "1";
    }
    else {
        print STDERR "ERROR: Nothing to do. Neither local or remote backup is enabled.\n\n";
	$now = localtime time;
	print STDERR "=================================================================\n";
	print STDERR "Aventurin{e} Backup on $host ($addr) \n";
	print STDERR "Backup aborted at: $now\n";
	print STDERR "=================================================================\n\n";
	system("/bin/rm -f /tmp/vz_vserver_backup.lock");
	exit 1;
    }
    if ($solbackupValues->{'MN_LocalBackupEnabled'} == "1") {
	$local_backup = "1";
        print STDERR "Local Backup is enabled on the master node.\n";
    }
    else {
        print STDERR "Local Backup is disabled on the master node.\n";
    }
    if ($solbackupValues->{'MN_RemoteBackupEnabled'} == "1") {
	$remote_backup = "1";
        print STDERR "Remote Backup is enabled on the master node.\n";
    }
    else {
        print STDERR "Remote Backup is disabled on the master node.\n";
    }
    print STDERR "\n";
}

#
## System preparation:
#

$local_target_dir = $solbackupValues->{'MN_BackupLocalLocation'};

# Sanitize local backup paths to remove trailing slashes and other traversal crap:
@ltarget_dir = ($local_target_dir);
$local_target_dir = File::Spec->catdir(@ltarget_dir);

# Create the local backup directory in case it is missing:
if(!-d $local_target_dir) {
	# Backup directory is missing - create it:
	system("/bin/mkdir $local_target_dir");
}
if (!-w $local_target_dir) {
	print STDERR "Specified backup location $solbackupValues->{'MN_BackupLocalLocation'} is not writeable!\n";
	print STDERR "Defaulting backup location to /vz/backup \n\n";
	$local_target_dir = "/vz/backup";
}

# Sanitize extra local backup paths to remove trailing slashes and other traversal crap:
$lt_vps_add = "/vps";
$lt_conf_add = "/conf";
$lt_aconf_add = "/aconf";
$lt_uconf_add = "/uconf";

@lvps_dir = ($local_target_dir, $lt_vps_add);
@lvps_conf = ($local_target_dir, $lt_conf_add);
@lvps_aconf = ($local_target_dir, $lt_aconf_add);
@lvps_uconf = ($local_target_dir, $lt_uconf_add);

$local_target_dir_vps = File::Spec->catdir(@lvps_dir);
$local_target_dir_conf = File::Spec->catdir(@lvps_conf);
$local_target_dir_aconf = File::Spec->catdir(@lvps_aconf);
$local_target_dir_uconf = File::Spec->catdir(@lvps_uconf);

if(!-d $local_target_dir_vps) {
        # Backup directory for VPS's is missing - create it:
        system("/bin/mkdir $local_target_dir_vps");
}
if(!-d $local_target_dir_conf) {
        # Backup directory for VPS's is missing - create it:
        system("/bin/mkdir $local_target_dir_conf");
}
if(!-d $local_target_dir_aconf) {
        # Backup directory for VPS's is missing - create it:
        system("/bin/mkdir $local_target_dir_aconf");
}
if(!-d $local_target_dir_uconf) {
        # Backup directory for VPS's customers is missing - create it:
        system("/bin/mkdir $local_target_dir_uconf");
}

#
## Run local backup
#

if ($local_backup == "1") {

    $local_target_dir_vps_bak = $local_target_dir_vps . '.bak'; 
    if(-d "$local_target_dir_vps_bak") {
        system("/bin/rm -R $local_target_dir_vps_bak");
    }
    if(-d $local_target_dir_vps) {
        system("/bin/mv $local_target_dir_vps $local_target_dir_vps_bak");
    }
    if(!-d $local_target_dir_vps) {
        system("/bin/mkdir $local_target_dir_vps");
    }

    # Find all VPS's where local backup is enabled:
    @oids = $cce->find('Vserver_Backup', {'VserverBackupLocalEnabled' => "1"});
    if ($#oids == -1) {
	print STDERR "No VPS has local backup enabled\n";
    }
    else {
	# Vserver_Backup found in CCE.
	foreach $line (@oids) {
            ($ok, $system) = $cce->get($line);
            $func = push(@vps_local, $system->{vpsid});
	}
	# How many VPS's with enabled local backup did we find?
	$num = $#vps_local;
	$anzahl = $num;
	$anzahl++;
	# Sort the IPs:
	@sorted_vps_local = sort { $a cmp $b } @vps_local;
	foreach $value (@sorted_vps_local) {
    	    if ($num > 0) {
        	$list .= $value . ";";
        	$vlist .= $value . ", ";
        	$num--;
    	    }
    	    else {
        	$list .= $value;
        	$vlist .= $value;
    	    }
	}
    }
    if ($#oids != -1) {
    	print STDERR "Found $anzahl VPS's where local backup is enabled. The VPS-IDS's are $vlist \n\n";
    }
    # Do the actual local backups:
    # Backup the backup config itself:
    $old_aconfig_files = "$local_target_dir_aconf/*.conf";
    system("/bin/rm -f $old_aconfig_files");
    if ( -d "/etc/vz/aconf") {
        system("/bin/cp /etc/vz/aconf/*.conf $local_target_dir_aconf/");
    }
    $old_uconfig_files = "$local_target_dir_uconf/*.*onf";
    system("/bin/rm -f $old_uconfig_files");
    if ( -f "/etc/vz/uconf/000_customers.conf") {
        system("/bin/cp /etc/vz/uconf/*.*onf $local_target_dir_uconf/");
    }
    # Backup the VPS's and their configs:
    $old_config_files = "$local_target_dir_conf/*.conf";
    system("/bin/rm -f $old_config_files");
    foreach $value (@sorted_vps_local) {
	$copy_conf = $local_target_dir_conf . "/$value" . '.conf';
        $copy_vps_bak =  $local_target_dir_vps_bak . "/$value";
	print STDERR "Performing local backup of VPS $value to $local_target_dir_vps\n";
	system("/bin/cp /etc/vz/conf/$value.conf $copy_conf");
	if(-d "$local_target_dir_vps_bak/$value") {	
		system("/bin/mv $copy_vps_bak $local_target_dir_vps");
	}
	system("/usr/bin/rsync -q --progress -var /vz/private/$value $local_target_dir_vps --delete-after");
	print STDERR "Finished local backup of VPS $value \n\n";
	# Store into CCE that we backed this VPS up:
	&localdone;
    }
    if(-d "$local_target_dir_vps_bak") {
        system("/bin/rm -R $local_target_dir_vps_bak");
    }
    if ($#oids != -1) {
	print STDERR "Local backup of VPS's $vlist finished! \n\n";
    }
}


#
## Run remote backup
#

if ($remote_backup == "1") {
    $vlist = "";
    $list = "";
    $value = "";
    # Find all VPS's where remote backup is enabled:
    @roids = $cce->find('Vserver_Backup', {'VserverBackupRemoteEnabled' => "1"});
    if ($#roids == -1) {
	print STDERR "No VPS has remote backup enabled\n";
    }
    else {
	# Vserver_Backup found in CCE.
	foreach $line (@roids) {
            ($ok, $system) = $cce->get($line);
            $func = push(@vps_remote, $system->{vpsid});
	}
	# How many VPS's with enabled remote backup did we find?
	$num = $#vps_remote;
	$anzahl = $num;
	$anzahl++;
	# Sort the IPs:
	@sorted_vps_remote = sort { $a cmp $b } @vps_remote;
	foreach $value (@sorted_vps_remote) {
    	    if ($num > 0) {
        	$list .= $value . ";";
        	$vlist .= $value . ", ";
        	$num--;
    	    }
    	    else {
        	$list .= $value;
        	$vlist .= $value;
    	    }
	}
    }
    if ($#roids != -1) {
    	print STDERR "Found $anzahl VPS's where remote backup is enabled. The VPS-IDS's are $vlist \n";
    }
    # Check if we have everything we need for the connection to the remote host:

    $username 	= $solbackupValues->{'MN_BackupRemoteTargetUser'};
    $remotehost = $solbackupValues->{'MN_BackupRemoteTargetHost'};
    $remoteport = $solbackupValues->{'MN_BackupRemoteTargetPort'};
    $remotepath = $solbackupValues->{'MN_BackupRemoteLocation'};
    $error 	= 0;

    if (!$username) {
    	print STDERR "ERROR: You did not specify a valid username for login to the remote host!\n\n";
	$error == 1;
    }
    if ($username != "root") {
    	print STDERR "ERROR: You are not using 'root' as username for the remote host! \n";
    	print STDERR "       All file ownerships will be totally messed up in the transfer! \n";
    	print STDERR "       That would leave you with an unrestoreable backup! \n\n";
	$error = 1;
    }
    if (!$remotehost) {
    	print STDERR "ERROR: You did not specify to which remote host you want to backup!\n\n";
	$error = 1;
    }
    if ((!$remoteport) || ($remoteport == "")) {
        print STDERR "WARNING: You did not specify the SSH port on the remote server. Defaulting to port 22.\n\n";
	$remoteport = "22";
    }
    if (!$remotepath) {
        print STDERR "ERROR: You did not specify the storage path on the remote server!\n\n";
	$error = 1;
    }

    if ($error == 0) {
        # Do the actual remote backups:
	foreach $value (@sorted_vps_remote) {
	    print STDERR "Performing remote backup of VPS $value\n\n";
	    print STDERR "Connecting to $remotehost:$remotepath at port $remoteport\n";

	    # On Target we may not have the subdirectries conf and vps, so we need to do a dirty
	    # work around to make sure that they are there. We temporarily create them locally, use
	    # SCP to copy them over and then remove them from local again. This doesn't cause any issues
	    # if the directories and/or older backup sets already exist on Target:
	    system("/bin/rm -Rf /tmp/vz_vserver_backup_temp");
	    system("/bin/mkdir /tmp/vz_vserver_backup_temp");
	    system("/bin/mkdir /tmp/vz_vserver_backup_temp/aconf");
	    system("/bin/mkdir /tmp/vz_vserver_backup_temp/uconf");
	    system("/bin/mkdir /tmp/vz_vserver_backup_temp/conf");
	    system("/bin/mkdir /tmp/vz_vserver_backup_temp/vps");
	    system("/usr/bin/scp -r -P $remoteport /tmp/vz_vserver_backup_temp/aconf $remotehost:$remotepath");
	    system("/usr/bin/scp -r -P $remoteport /tmp/vz_vserver_backup_temp/uconf $remotehost:$remotepath");
	    system("/usr/bin/scp -r -P $remoteport /tmp/vz_vserver_backup_temp/conf $remotehost:$remotepath");
	    system("/usr/bin/scp -r -P $remoteport /tmp/vz_vserver_backup_temp/vps $remotehost:$remotepath");
	    system("/bin/rm -R /tmp/vz_vserver_backup_temp");

	    # Back up aconfig files:
    	    if ( -d "/etc/vz/aconf") {
	    	system("/usr/bin/rsync -q --progress -var -e 'ssh -p $remoteport' /etc/vz/aconf/*.conf $remotehost:$remotepath/aconf/ --delete-after");
	    }
	    # Back up uconfig files:
    	    if ( -f "/etc/vz/uconf/000_customers.conf") {
	    	system("/usr/bin/rsync -q --progress -var -e 'ssh -p $remoteport' /etc/vz/uconf/*.*onf $remotehost:$remotepath/uconf/ --delete-after");
	    }
	    # Back up config files:
	    system("/bin/touch /etc/vz/conf/$value.conf");
	    system("/usr/bin/rsync -q --progress -var -e 'ssh -p $remoteport' /etc/vz/conf/$value.conf $remotehost:$remotepath/conf/ --delete-after");
	    # Back up VPS:
	    system("/usr/bin/rsync -q --progress -var -e 'ssh -p $remoteport' /vz/private/$value $remotehost:$remotepath/vps/ --delete-after");

	    print STDERR "Finished remote backup of VPS $value \n\n";
	    &remotedone;
	}
	if ($#roids != -1) {
	    print STDERR "Remote backup of VPS's $vlist finished! \n\n";
	}
    }
    else {
        print STDERR "WARNING: Remote backup was not performed.\n\n";
    }
}

$cce->bye('SUCCESS');

$now = localtime time;

print STDERR "=================================================================\n";
print STDERR "Aventurin{e} Backup on $host ($addr) \n";
print STDERR "Backup finished at: $now\n";
print STDERR "=================================================================\n\n";

system("/bin/rm -f /tmp/vz_vserver_backup.lock");


sub localdone {

    $stamp = localtime time;

    ($sys_oid) = $cce->find('Vserver_Backup', {'vpsid' => $value});
    ($ok, $sysvs) = $cce->get($sys_oid);
    ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
        'vpsid' => $value,
        'LBpresent' => '1',
        'LBtimestamp' => $stamp
    });
}

sub remotedone {

    $stamp = localtime time;

    ($sys_oid) = $cce->find('Vserver_Backup', {'vpsid' => $value});
    ($ok, $sysvs) = $cce->get($sys_oid);
    ($ok) = $cce->set($sys_oid, '',{        # beware: CLASS vs. Namespace (see schema)! Our NameSpace is empty, hence: ''
        'vpsid' => $value,
        'LRpresent' => '1',
        'LRtimestamp' => $stamp
    });
}

exit 0;
